@@ -219,14 +219,14 @@ subst:
                 case SUBST_MAJOR: {
                         char num[UTIL_PATH_SIZE];
 
-                        sprintf(num, "%d", major(udev_device_get_devnum(dev)));
+                        sprintf(num, "%u", major(udev_device_get_devnum(dev)));
                         l = strpcpy(&s, l, num);
                         break;
                 }
                 case SUBST_MINOR: {
                         char num[UTIL_PATH_SIZE];
 
-                        sprintf(num, "%d", minor(udev_device_get_devnum(dev)));
+                        sprintf(num, "%u", minor(udev_device_get_devnum(dev)));
                         l = strpcpy(&s, l, num);
                         break;
                 }
@@ -388,7 +388,7 @@ static int spawn_exec(struct udev_event *event,
                 if (fd_stderr < 0)
                         dup2(fd, STDERR_FILENO);
         } else
-                log_error("open /dev/null failed: %m");
+                log_error_errno(errno, "open /dev/null failed: %m");
 
         /* connect pipes to std{out,err} */
         if (fd_stdout >= 0) {
@@ -410,7 +410,7 @@ static int spawn_exec(struct udev_event *event,
         execve(argv[0], argv, envp);
 
         /* exec failed */
-        log_error("failed to execute '%s' '%s': %m", argv[0], cmd);
+        log_error_errno(errno, "failed to execute '%s' '%s': %m", argv[0], cmd);
 
         return -errno;
 }
@@ -438,14 +438,14 @@ static void spawn_read(struct udev_event *event,
 
         fd_ep = epoll_create1(EPOLL_CLOEXEC);
         if (fd_ep < 0) {
-                log_error("error creating epoll fd: %m");
+                log_error_errno(errno, "error creating epoll fd: %m");
                 return;
         }
 
         if (fd_stdout >= 0) {
                 r = epoll_ctl(fd_ep, EPOLL_CTL_ADD, fd_stdout, &ep_outpipe);
                 if (r < 0) {
-                        log_error("fail to add stdout fd to epoll: %m");
+                        log_error_errno(errno, "fail to add stdout fd to epoll: %m");
                         return;
                 }
         }
@@ -453,7 +453,7 @@ static void spawn_read(struct udev_event *event,
         if (fd_stderr >= 0) {
                 r = epoll_ctl(fd_ep, EPOLL_CTL_ADD, fd_stderr, &ep_errpipe);
                 if (r < 0) {
-                        log_error("fail to add stderr fd to epoll: %m");
+                        log_error_errno(errno, "fail to add stderr fd to epoll: %m");
                         return;
                 }
         }
@@ -482,7 +482,7 @@ static void spawn_read(struct udev_event *event,
                 if (fdcount < 0) {
                         if (errno == EINTR)
                                 continue;
-                        log_error("failed to poll: %m");
+                        log_error_errno(errno, "failed to poll: %m");
                         return;
                 } else if (fdcount == 0) {
                         log_error("timeout '%s'", cmd);
@@ -510,7 +510,7 @@ static void spawn_read(struct udev_event *event,
                                                 memcpy(&result[respos], buf, count);
                                                 respos += count;
                                         } else {
-                                                log_error("'%s' ressize %zd too short", cmd, ressize);
+                                                log_error("'%s' ressize %zu too short", cmd, ressize);
                                         }
                                 }
 
@@ -528,7 +528,7 @@ static void spawn_read(struct udev_event *event,
                         } else if (ev[i].events & EPOLLHUP) {
                                 r = epoll_ctl(fd_ep, EPOLL_CTL_DEL, *fd, NULL);
                                 if (r < 0) {
-                                        log_error("failed to remove fd from epoll: %m");
+                                        log_error_errno(errno, "failed to remove fd from epoll: %m");
                                         return;
                                 }
                                 *fd = -1;
@@ -577,22 +577,22 @@ static int spawn_wait(struct udev_event *event,
                         if (errno == EINTR)
                                 continue;
                         err = -errno;
-                        log_error("failed to poll: %m");
+                        log_error_errno(errno, "failed to poll: %m");
                         goto out;
                 }
                 if (fdcount == 0) {
-                        log_warning("slow: '%s' [%u]", cmd, pid);
+                        log_warning("slow: '%s' ["PID_FMT"]", cmd, pid);
 
                         fdcount = poll(pfd, 1, timeout);
                         if (fdcount < 0) {
                                 if (errno == EINTR)
                                         continue;
                                 err = -errno;
-                                log_error("failed to poll: %m");
+                                log_error_errno(errno, "failed to poll: %m");
                                 goto out;
                         }
                         if (fdcount == 0) {
-                                log_error("timeout: killing '%s' [%u]", cmd, pid);
+                                log_error("timeout: killing '%s' ["PID_FMT"]", cmd, pid);
                                 kill(pid, SIGKILL);
                         }
                 }
@@ -614,20 +614,20 @@ static int spawn_wait(struct udev_event *event,
                                 if (waitpid(pid, &status, WNOHANG) < 0)
                                         break;
                                 if (WIFEXITED(status)) {
-                                        log_debug("'%s' [%u] exit with return code %i", cmd, pid, WEXITSTATUS(status));
+                                        log_debug("'%s' ["PID_FMT"] exit with return code %i", cmd, pid, WEXITSTATUS(status));
                                         if (WEXITSTATUS(status) != 0)
                                                 err = -1;
                                 } else if (WIFSIGNALED(status)) {
-                                        log_error("'%s' [%u] terminated by signal %i (%s)", cmd, pid, WTERMSIG(status), strsignal(WTERMSIG(status)));
+                                        log_error("'%s' ["PID_FMT"] terminated by signal %i (%s)", cmd, pid, WTERMSIG(status), strsignal(WTERMSIG(status)));
                                         err = -1;
                                 } else if (WIFSTOPPED(status)) {
-                                        log_error("'%s' [%u] stopped", cmd, pid);
+                                        log_error("'%s' ["PID_FMT"] stopped", cmd, pid);
                                         err = -1;
                                 } else if (WIFCONTINUED(status)) {
-                                        log_error("'%s' [%u] continued", cmd, pid);
+                                        log_error("'%s' ["PID_FMT"] continued", cmd, pid);
                                         err = -1;
                                 } else {
-                                        log_error("'%s' [%u] exit with status 0x%04x", cmd, pid, status);
+                                        log_error("'%s' ["PID_FMT"] exit with status 0x%04x", cmd, pid, status);
                                         err = -1;
                                 }
                                 pid = 0;
@@ -677,7 +677,6 @@ int udev_event_spawn(struct udev_event *event,
                      usec_t timeout_warn_usec,
                      const char *cmd, char **envp, const sigset_t *sigmask,
                      char *result, size_t ressize) {
-        struct udev *udev = event->udev;
         int outpipe[2] = {-1, -1};
         int errpipe[2] = {-1, -1};
         pid_t pid;
@@ -690,17 +689,17 @@ int udev_event_spawn(struct udev_event *event,
         udev_build_argv(event->udev, arg, NULL, argv);
 
         /* pipes from child to parent */
-        if (result != NULL || udev_get_log_priority(udev) >= LOG_INFO) {
+        if (result != NULL || log_get_max_level() >= LOG_INFO) {
                 if (pipe2(outpipe, O_NONBLOCK) != 0) {
                         err = -errno;
-                        log_error("pipe failed: %m");
+                        log_error_errno(errno, "pipe failed: %m");
                         goto out;
                 }
         }
-        if (udev_get_log_priority(udev) >= LOG_INFO) {
+        if (log_get_max_level() >= LOG_INFO) {
                 if (pipe2(errpipe, O_NONBLOCK) != 0) {
                         err = -errno;
-                        log_error("pipe failed: %m");
+                        log_error_errno(errno, "pipe failed: %m");
                         goto out;
                 }
         }
@@ -737,7 +736,7 @@ int udev_event_spawn(struct udev_event *event,
 
                 _exit(2 );
         case -1:
-                log_error("fork of '%s' failed: %m", cmd);
+                log_error_errno(errno, "fork of '%s' failed: %m", cmd);
                 err = -1;
                 goto out;
         default:
@@ -772,225 +771,92 @@ out:
         return err;
 }
 
-#ifdef ENABLE_RULE_GENERATOR
-/* function to return the count of rules that assign NAME= to a value matching arg#2 , defined in udev-rules.c */
-int udev_rules_assigning_name_to(struct udev_rules *rules,const char *match_name);
-#endif
-
-static int rename_netif_dev_fromname_toname(struct udev_device *dev,const char *oldname,const char *newname) {
+static int rename_netif(struct udev_event *event) {
+        struct udev_device *dev = event->dev;
+        char name[IFNAMSIZ];
+        const char *oldname;
+        int r;
         int sk;
         struct ifreq ifr;
-        int err;
+
+        oldname = udev_device_get_sysname(dev);
+
+        strscpy(name, IFNAMSIZ, event->name);
 
         sk = socket(PF_INET, SOCK_DGRAM, 0);
-        if (sk < 0) {
-                err = -errno;
-                log_error("error opening socket: %m\n");
-                return err;
-        }
+        if (sk < 0)
+                return log_error_errno(-errno, "error opening socket: %m");
 
         memzero(&ifr, sizeof(struct ifreq));
-
         strscpy(ifr.ifr_name, IFNAMSIZ, oldname);
-        strscpy(ifr.ifr_newname, IFNAMSIZ, newname);
-        err = ioctl(sk, SIOCSIFNAME, &ifr);
+        strscpy(ifr.ifr_newname, IFNAMSIZ, name);
+        r = ioctl(sk, SIOCSIFNAME, &ifr);
+        if (r < 0)
+                return log_error_errno(-errno, "Error changing net interface name '%s' to '%s': %m", oldname, name);
 
-#ifdef ENABLE_RULE_GENERATOR
-        int loop;
-
-        if (err == 0) {
-                log_info("renamed network interface %s to %s\n", ifr.ifr_name, ifr.ifr_newname);
-                goto out;
-        }
-        /* keep trying if the destination interface name already exists */
-        log_debug("collision on rename of network interface %s to %s , retrying until timeout\n",
-                ifr.ifr_name, ifr.ifr_newname);
-
-        err = -errno;
-        if (err != -EEXIST)
-                goto out;
-
-        /* wait a maximum of 90 seconds for our target to become available */
-        loop = 90 * 20;
-        while (loop--) {
-                const struct timespec duration = { 0, 1000 * 1000 * 1000 / 20 };
-
-                nanosleep(&duration, NULL);
-
-                err = ioctl(sk, SIOCSIFNAME, &ifr);
-                if (err == 0) {
-                        log_info("renamed network interface %s to %s\n", ifr.ifr_name, ifr.ifr_newname);
-                        break;
-                }
-                err = -errno;
-                if (err != -EEXIST)
-                        break;
-        }
-
-out:
-        if (err < 0)
-                log_error("error changing net interface name %s to %s: %m\n", ifr.ifr_name, ifr.ifr_newname);
-#else
-        if (err >= 0) {
-                log_info("renamed network interface %s to %s\n", ifr.ifr_name, ifr.ifr_newname);
-        } else {
-                err = -errno;
-                log_error("error changing net interface name %s to %s: %m\n", ifr.ifr_name, ifr.ifr_newname);
-        }
-#endif
+        log_debug("renamed network interface '%s' to '%s'", oldname, name);
 
         close(sk);
-        return err;
-}
-
-static int rename_netif(struct udev_event *event) {
-        return rename_netif_dev_fromname_toname(event->dev,udev_device_get_sysname(event->dev),event->name);
+        return 0;
 }
 
 void udev_event_execute_rules(struct udev_event *event,
-                              usec_t timeout_usec,
-                              usec_t timeout_warn_usec,
-                              struct udev_rules *rules, const sigset_t *sigmask) {
+                              usec_t timeout_usec, usec_t timeout_warn_usec,
+                              struct udev_list *properties_list,
+                              struct udev_rules *rules,
+                              const sigset_t *sigmask) {
         struct udev_device *dev = event->dev;
 
         if (udev_device_get_subsystem(dev) == NULL)
                 return;
 
         if (streq(udev_device_get_action(dev), "remove")) {
-                udev_device_read_db(dev, NULL);
-                udev_device_delete_db(dev);
-                udev_device_tag_index(dev, NULL, false);
-
                 if (major(udev_device_get_devnum(dev)) != 0)
                         udev_watch_end(event->udev, dev);
 
-                udev_rules_apply_to_event(rules, event, timeout_usec, timeout_warn_usec, sigmask);
+                udev_rules_apply_to_event(rules, event,
+                                          timeout_usec, timeout_warn_usec,
+                                          properties_list,
+                                          sigmask);
 
                 if (major(udev_device_get_devnum(dev)) != 0)
                         udev_node_remove(dev);
-        } else {
-                event->dev_db = udev_device_new(event->udev);
-                if (event->dev_db != NULL) {
-                        udev_device_set_syspath(event->dev_db, udev_device_get_syspath(dev));
-                        udev_device_set_subsystem(event->dev_db, udev_device_get_subsystem(dev));
-                        udev_device_set_devnum(event->dev_db, udev_device_get_devnum(dev));
-                        udev_device_read_db(event->dev_db, NULL);
-                        udev_device_set_info_loaded(event->dev_db);
 
+                udev_device_delete_db(dev);
+                udev_device_tag_index(dev, NULL, false);
+        } else {
+                event->dev_db = udev_device_clone_with_db(dev);
+                if (event->dev_db != NULL) {
                         /* disable watch during event processing */
                         if (major(udev_device_get_devnum(dev)) != 0)
                                 udev_watch_end(event->udev, event->dev_db);
                 }
 
                 if (major(udev_device_get_devnum(dev)) == 0 &&
-                    streq(udev_device_get_action(dev), "move")) {
-                        struct udev_list_entry *entry;
+                    streq(udev_device_get_action(dev), "move"))
+                        udev_device_copy_properties(dev, event->dev_db);
 
-                        for ((entry = udev_device_get_properties_list_entry(event->dev_db)); entry; entry = udev_list_entry_get_next(entry)) {
-                                const char *key, *value;
-                                struct udev_list_entry *property;
-
-                                key = udev_list_entry_get_name(entry);
-                                value = udev_list_entry_get_value(entry);
-
-                                property = udev_device_add_property(event->dev, key, value);
-                                udev_list_entry_set_num(property, true);
-                        }
-                }
-
-                udev_rules_apply_to_event(rules, event, timeout_usec, timeout_warn_usec, sigmask);
+                udev_rules_apply_to_event(rules, event,
+                                          timeout_usec, timeout_warn_usec,
+                                          properties_list,
+                                          sigmask);
 
                 /* rename a new network interface, if needed */
-
-                /* ENABLE_RULE_GENERATOR conditional:
-                 * if this is a net iface, and it is an add event,
-                 * and as long as all of the following are FALSE:
-                 *  - no NAME target and the current name is not being used
-                 *  - there is a NAME target and it is the same as the current name
-                 *  - the rules can successfully be searched for the current name (not really part of the conditional)
-                 * the run the rename.
-                 *
-                 * note - udev_rules_assigning_name_to is run when event->name is NULL to ensure renames happen,
-                 * but also on its own to check if a temp-rename is necessary when event->name exists. 
-                 *
-                 * A temp-rename is necessary when:
-                 * - there is no rule renaming the current iface but the current name IS used in some other rule
-                 * - there is a rule renaming the current iface, 
-                 *   the current name IS used AND the target name != the current name
-                 */
-
-                int r;
                 if (udev_device_get_ifindex(dev) > 0 && streq(udev_device_get_action(dev), "add") &&
-#ifdef ENABLE_RULE_GENERATOR
-                    (event->name == NULL && (r=udev_rules_assigning_name_to(rules,udev_device_get_sysname(dev))) > 0 ||
-                    event->name != NULL && !streq(event->name, udev_device_get_sysname(dev)))) {
-#else
                     event->name != NULL && !streq(event->name, udev_device_get_sysname(dev))) {
-#endif
-                        char syspath[UTIL_PATH_SIZE];
-                        char *pos;
-                        char *finalifname = event->name;
-#ifdef ENABLE_RULE_GENERATOR
-                        char newifname[IFNAMSIZ];
-
-                        /* r is the number of rules that assign a device with NAME= this sysname */
-                        if (r > 0 || (r=udev_rules_assigning_name_to(rules,udev_device_get_sysname(dev))) > 0) { 
-                                /* have a conflict, rename to a temp name */
-                                char *newpos;
-                                int ifidnum;
-
-                                /* build the temporary iface name */
-                                strscpy(newifname, IFNAMSIZ, udev_device_get_sysname(dev));
-                                newpos=pos=&newifname[strcspn(newifname,"0123456789")];
-                                ifidnum=(int)strtol(pos,&newpos,10);
-                                *pos='\0';
-                                if (newpos > pos && *newpos == '\0') /* append new iface num to name */
-                                        /* use udev_device_get_ifindex(dev) as it is unique to every iface */
-                                        snprintf(pos,IFNAMSIZ+(newifname-pos), "%d", 128 - udev_device_get_ifindex(dev));
-
-                                /* note, r > 0, which will skip the post-rename stuff if no rename occurs */
-
-                                /* if sysname isn't already the tmpname (ie there is no numeric component), do the rename */
-                                if (!streq(newifname,udev_device_get_sysname(dev))) {
-                                        r = rename_netif_dev_fromname_toname(dev,udev_device_get_sysname(dev),newifname);
-                                        if (r == 0) {
-                                                finalifname = newifname;
-                                                log_debug("renamed netif to '%s' for collision avoidance\n", newifname);
-                                        } else {
-                                                log_error("could not rename netif to '%s' for collision avoidance\n",newifname);
-                                        }
-                                }
-                                /* rename it now to its final target if its not already there */
-                                if (event->name != NULL && !streq(event->name, newifname)) {
-                                        r = rename_netif_dev_fromname_toname(dev,newifname,event->name);
-                                        if (r == 0)
-                                                finalifname = event->name;
-				}
-
-                        } else { /* no need to rename to a tempname first, do a regular direct rename to event->name */
-
-                                r = 1; /* skip the post-rename stuff if no rename occurs */
-                                if (!streq(event->name, udev_device_get_sysname(dev)))
-                                        r = rename_netif(event);
-                        }
-#else
+                        int r;
 
                         r = rename_netif(event);
-#endif
-                        if (r >= 0) {
-                                /* remember old name */
-                                udev_device_add_property(dev, "INTERFACE_OLD", udev_device_get_sysname(dev));
-
-                                /* now change the devpath, because the kernel device name has changed */
-                                strscpy(syspath, sizeof(syspath), udev_device_get_syspath(dev));
-                                pos = strrchr(syspath, '/');
-                                if (pos != NULL) {
-                                        pos++;
-                                        strscpy(pos, sizeof(syspath) - (pos - syspath), finalifname);
-                                        udev_device_set_syspath(event->dev, syspath);
-                                        udev_device_add_property(dev, "INTERFACE", udev_device_get_sysname(dev));
+                        if (r < 0)
+                                log_warning_errno(r, "could not rename interface '%d' from '%s' to '%s': %m", udev_device_get_ifindex(dev),
+                                                  udev_device_get_sysname(dev), event->name);
+                        else {
+                                r = udev_device_rename(dev, event->name);
+                                if (r < 0)
+                                        log_warning_errno(r, "renamed interface '%d' from '%s' to '%s', but could not update udev_device: %m",
+                                                          udev_device_get_ifindex(dev), udev_device_get_sysname(dev), event->name);
+                                else
                                         log_debug("changed devpath to '%s'", udev_device_get_devpath(dev));
-                                }
                         }
                 }
 
@@ -1025,18 +891,14 @@ void udev_event_execute_rules(struct udev_event *event,
                 }
 
                 /* preserve old, or get new initialization timestamp */
-                if (event->dev_db != NULL && udev_device_get_usec_initialized(event->dev_db) > 0)
-                        udev_device_set_usec_initialized(event->dev, udev_device_get_usec_initialized(event->dev_db));
-                else if (udev_device_get_usec_initialized(event->dev) == 0)
-                        udev_device_set_usec_initialized(event->dev, now(CLOCK_MONOTONIC));
+                udev_device_ensure_usec_initialized(event->dev, event->dev_db);
 
                 /* (re)write database file */
-                udev_device_update_db(dev);
                 udev_device_tag_index(dev, event->dev_db, true);
+                udev_device_update_db(dev);
                 udev_device_set_is_initialized(dev);
 
-                udev_device_unref(event->dev_db);
-                event->dev_db = NULL;
+                event->dev_db = udev_device_unref(event->dev_db);
         }
 }
 
